;; ------------------
;; variables globales
;; ------------------
(setf caderro "")
(setf eleccion 'aleatorio)

;; ---------------------------------
;; cargar los ejemplos de laberintos
;; ---------------------------------
(load "ejemplos.lsp")

;; -----------------------------------------------
;; lista de asociación para imprimir el movimiento
;; dentro del laberinto
;; -----------------------------------------------
(setf flechas-movimiento '(
			   ((0 1) "⇒")
			   ((0 -1) "⇐")
			   ((1 0) "⇓")
			   ((-1 0) "⇑")
			   )
      )

(setf flechas2 '(
			   (1 " ")
			   (3 "⇒")
			   (2 "⇐")
			   (4 "⇓")
			   (5 "⇑")
			   )
      )

(setf direccion-numero '(
			   ((0 1) 3)
			   ((0 -1) 2)
			   ((1 0) 4)
			   ((-1 0) 5)
			   )
      )
;; ----------------------------------
;; cadenas para imprimir el laberinto
;; ----------------------------------
(setf string1 (make-array 2 :initial-contents '("+" " ")))
(setf string2 (make-array 2 :initial-contents '("---" "   ")))

(defun primero (lista) (first lista))
(defun segundo (lista) (second lista))
(defun tercero (lista) (third lista))

;; el laberinto está resuelto cuando estoy en una posición
;; abierta en el contorno y distinta de la original
;; m es el número de filas y n el de columnas
(defun esta-resuelto-labp (pos-actual pos-original m n)
  (let (
	(x (first pos-actual))
	(y (second pos-actual))
	)
    (and
     (or (= x 0) (= x (- m 1)) (= y 0) (= y (- n 1)))
     (not (equal pos-actual pos-original))
     )
    )
  )

;; posicion = (x y)
;; m es el número de filas y n el de columnas de la matriz
(defun es-coordenada-validap (posicion m n)
  (let (
	(x (first posicion))
	(y (second posicion))
	)
    (and (>= x 0) (< x m) (>= y 0) (< y n))
    )
  )


;; devuelve la lista de adyacentes a posicion no visitados.
;; lvisit es una lista con las posiciones visitadas
;; laberinto es la matriz bidimensional del laberinto
(defun lista-de-adyacentes-no-visitados (posicion lvisit laberinto)
  (let (
	(x (first posicion))
	(y (second posicion))
	(l '())
	(u 0)
	(v 0)
	(result '())
	(m 0)
	(n 0)
	)
    ;; calcular el número de filas y columnas del laberinto
    (setf u (array-dimensions laberinto))
    (setf m (first u)) ;; número de filas
    (setf n (second u)) ;; número de columnas
    ;; construir la lista de posibles posiciones
    (setf l
	  (list
	   (list (- x 1) y)
	   (list x (- y 1))
	   (list x (+ y 1))
	   (list (+ x 1) y)
	   )
	  )
    (dolist (u l result)
      (when (es-coordenada-validap u m n)
	(when (not (member u lvisit :test #'equal)) ;; no se ha visitado
	  (setf v (aref laberinto (first u) (second u)))
	  (when (= v 1) ;; éste vale
	    (setf result (cons u result))
	    )
	  )
	)
      )
    )
  )

(defun solucion-laberinto (laberinto pos-original recorrido)
  (let (
	(actual pos-original)
	(lvisit '())
	(camino '())
	(ind-salida nil) ;; cuando sea t, salir sin condiciones
	(inde nil) ;; indicador de éxito o fracaso
	(x '())
	(u '())
	(m 0)
	(n 0)
	)
    (setf u (array-dimensions laberinto))
    (setf m (first u)) ;; número de filas
    (setf n (second u)) ;; número de columnas
    (setf lvisit (cons actual lvisit))
    (setf camino (cons actual camino))
    (loop while (not ind-salida) do ;; mientras haya algo que hacer
	  (cond
	   ((esta-resuelto-labp actual pos-original m n)
	    (setf ind-salida t)
	    (setf inde t)
	    )
	   (t ;; no resuelto, hay que trabajar. Busca los adyacentes a actual no visitados
;;	    (format t "~a~%" camino)
	    (setf x (lista-de-adyacentes-no-visitados actual lvisit laberinto))
	    (cond
	     ((endp x) ;; no hay ninguno, hay que retroceder
	      (pop camino) ;; quitar el último de camino
	      (cond
	       ((endp camino) ;; no se puede retroceder, problema sin solución
		(setf ind-salida t)
		(setf inde nil)
		)
	       (t ;; retroceso propiamente dicho
		(setf actual (first camino)) ;; retroceder
		)
	       )
	      )
	     (t ;; lista de adyacentes no vacía
	      (if (eq recorrido 'aleatorio)
		  (setf u (let* (
				 (l (length x))
				 (i (aleat 0 (- l 1)))
				 )
			    (primero (nthcdr i x))
			    )
			)
		(setf u (first x)) ;; cuando no es aleatorio
		)
	      (setf lvisit (cons u lvisit))
	      (setf camino (cons u camino))
	      (setf actual u)
	      )
	     )
	    )
	   )
	  ) ;; fin de loop
    (if inde camino '())
    ) ;; fin de let
  ) ;; fin de solucion-laberinto

(defun imprime-laberinto (laberinto)
  (let* (
	 (p (array-dimensions laberinto))
	 (m (first p))
	 (n (second p))
	 (result "   ")
	 (cadena "")
	 (u 0)
	 (v 0)
	 (w 0)
	 (r 0)
	 )
    ;; la cabecera numérica
    (dotimes (i n result)
      (setf result (concatenate 'string result (format nil "  ~2,'0D" i)))
      )
    (format t "~a~%" result)
    ;; ahora el resto
    (dotimes (i m)
      ;; la primera fila
      (setf result "   ")
      (dotimes (j n result)
	(cond
	 ((and (zerop i) (zerop j))
	  (setf w (aref laberinto i j))
	  (setf u (aref string1 w))
	  (setf v (aref string2 w))
	  (setf cadena (concatenate 'string u v))
	  )
	 ((and (zerop i) (plusp j))
	  (setf u (* (aref laberinto i j) (aref laberinto i (- j 1))))
	  (setf cadena
		(concatenate 'string
			     (aref string1 u)
			     (aref string2 (aref laberinto i j))
			     )
		)
	  )
	 ((and (plusp i) (zerop j))
	  (setf u (* (aref laberinto i j) (aref laberinto (- i 1) j)))
	  (setf cadena
		(concatenate 'string
			     (aref string1 u)
			     (aref string2 u)
			     )
		)
	  )
	 (t ;; i > 0 y j > 0
	  (setf u (*
		   (aref laberinto i j)
		   (aref laberinto (- i 1) j)
		   (aref laberinto i (- j 1))
		   (aref laberinto (- i 1) (- j 1))
		   )
		)
	  (setf v (*
		   (aref laberinto i j)
		   (aref laberinto (- i 1) j)
		   )
		)
	  (setf cadena
		(concatenate 'string
			     (aref string1 u)
			     (aref string2 v)
			     )
		)
	  )
	 ) ;; fin del cond
	(setf result (concatenate 'string result cadena)) ;; ir acumulando
	) ;; fin del dotimes j
      ;; el cierre de columna
      (setf r 1)
      (when (> i 0)
	(setf r (aref laberinto (- i 1) (- n 1)))
	)
      (setf v (* (aref laberinto i (- n 1)) r))
      (format t "~a~a~%" result (aref string1 v))
      ;; lista la primera fila. Ahora la segunda

      (setf result (format nil "~2,'0D " i))
      (dotimes (j n result)
	(cond
	 ((zerop (aref laberinto i j))
	  (setf cadena "|///")
	  )
	 (t
	  (cond
	   ((zerop j)
	    (setf cadena "    ")
	    )
	   (t ;; j > 0
	    (if (zerop (aref laberinto i (- j 1)))
		(setf cadena "|   ")
	      (setf cadena "    ")
	      )
	    )
	   )
	  )
	 )
	(setf result (concatenate 'string result cadena)) ;; ir acumulando
	) ;; fin del dotimes j
      (setf cadena (if (zerop (aref laberinto i (- n 1))) "|" " "))
      (format t "~a~a~%" result cadena)
      ) ;; fin del dotimes i
    ;; la última fila (cierre del laberinto)
    (setf result "   ")
    (dotimes (j n result)
      (cond
       ((zerop j)
	(setf u (aref laberinto (- m 1) j))
	(setf cadena (concatenate
		      'string
		      (aref string1 u)
		      (aref string2 u)
		      )
	      )
	)
       (t ;; j > 0
	(setf u (*
		 (aref laberinto (- m 1) j)
		 (aref laberinto (- m 1) (- j 1))
		 )
	      )
	(setf v (aref laberinto (- m 1) j))
	(setf cadena
	      (concatenate 'string
			   (aref string1 u)
			   (aref string2 v)
			   )
	      )
	)
       )
      (setf result (concatenate 'string result cadena)) ;; ir acumulando
      ) ;; fin del dotimes j n ..
    (format t "~a~a~%" result (aref string1 (aref laberinto (- m 1) (- n 1))))
    ) ;; fin del let*
  ) ;; fin de la función


(defun dif-pares (p1 p2)
  (list
   (- (first p1) (first p2))
   (- (second p1) (second p2))
   )
  )

;; Hasta que no encontremos una función que copie una matriz, utilizar
;; esta función rudimentaria. No vale la inocente:
;; (setf nueva-matriz antigua-matriz)
;; ya que cualquier cambio en una de ellas se refleja en la otra
(defun copia-matriz (matriz)
  (let* (
	 (p (array-dimensions matriz))
	 (m (first p)) ;; número filas
	 (n (second p)) ;; número de columnas
	 (result (make-array (list m n)))
	 )
    (dotimes (i m)
      (dotimes (j n)
	(setf (aref result i j) (aref matriz i j))
	)
      )
    result
    )
  )

(defun normaliza-valor (numero)
  (if (zerop numero) 0 1)
  )

(defun imprime-laberinto-recorrido (laberinto solucion)
  (let* (
	 (p (array-dimensions laberinto))
	 (m (first p))
	 (n (second p))
	 (result "   ")
	 (cadena "")
	 (u 0)
	 (v 0)
	 (w 0)
	 (r 0)
	 (x '())
	 (y '())
	 (direccion '())
	 (solu (reverse solucion))
	 (labm (copia-matriz laberinto)) ;; en esta matriz manejamos los movimientos
	 )
    ;; Escribir en el laberinto los movimientos
    (loop while (not (endp solu)) do ;; mientras haya algo que hacer
	  (setf x (primero solu))
	  (when (not (endp (rest solu)))
	    (setf y (segundo solu))
	    (setf direccion (dif-pares y x))
	    )
	  (setf (aref labm (primero x) (segundo x))
		(segundo (assoc direccion direccion-numero :test #'equal))
		)
	  (setf solu (rest solu))
	  )
    ;; copiada. Ahora igual que imprime-laberinto
    ;; la cabecera numérica
    (dotimes (i n result)
      (setf result (concatenate 'string result (format nil "  ~2,'0D" i)))
      )
    (format t "~a~%" result)
    ;; ahora el resto
    (dotimes (i m)
      ;; la primera fila
      (setf result "   ")
      (dotimes (j n result)
	(cond
	 ((and (zerop i) (zerop j))
	  (setf w (aref laberinto i j))
	  (setf u (aref string1 w))
	  (setf v (aref string2 w))
	  (setf cadena (concatenate 'string u v))
	  )
	 ((and (zerop i) (plusp j))
	  (setf u (* (aref laberinto i j) (aref laberinto i (- j 1))))
	  (setf cadena
		(concatenate 'string
			     (aref string1 u)
			     (aref string2 (aref laberinto i j))
			     )
		)
	  )
	 ((and (plusp i) (zerop j))
	  (setf u (* (aref laberinto i j) (aref laberinto (- i 1) j)))
	  (setf cadena
		(concatenate 'string
			     (aref string1 u)
			     (aref string2 u)
			     )
		)
	  )
	 (t ;; i > 0 y j > 0
	  (setf u (*
		   (aref laberinto i j)
		   (aref laberinto (- i 1) j)
		   (aref laberinto i (- j 1))
		   (aref laberinto (- i 1) (- j 1))
		   )
		)
	  (setf v (*
		   (aref laberinto i j)
		   (aref laberinto (- i 1) j)
		   )
		)
	  (setf cadena
		(concatenate 'string
			     (aref string1 u)
			     (aref string2 v)
			     )
		)
	  )
	 ) ;; fin del cond
	(setf result (concatenate 'string result cadena)) ;; ir acumulando
	) ;; fin del dotimes j
      ;; el cierre de columna
      (setf r 1)
      (when (> i 0)
	(setf r (aref laberinto (- i 1) (- n 1)))
	)
      (setf v (* (aref laberinto i (- n 1)) r))
      (format t "~a~a~%" result (aref string1 v))
      ;; lista la primera fila. Ahora la segunda

      (setf result (format nil "~2,'0D " i))
      (dotimes (j n result)
	(setf r (aref labm i j))
	(when (> r 0)
	  (setf u (segundo (assoc r flechas2)))
	  )
	(cond
	 ((zerop r)
	  (setf cadena "|///")
	  )
	 (t
	  (cond
	   ((zerop j)
	    (setf cadena (format nil " ~a  " u))
	    )
	   (t ;; j > 0
	    (if (zerop (aref labm i (- j 1)))
		(setf cadena (format nil "| ~a " u))
	      (setf cadena (format nil "  ~a " u))
	      )
	    )
	   )
	  )
	 )
	(setf result (concatenate 'string result cadena)) ;; ir acumulando
	) ;; fin del dotimes j
;;      (format t "Aquí he llegado~%")
;;      (format t "result = \"~a\"~%" result)
      (setf cadena (if (zerop (aref laberinto i (- n 1))) "|" " "))
      (format t "~a~a~%" result cadena)
;;      (format t "Aquí también he llegado~%")
      ) ;; fin del dotimes i
    ;; la última fila (cierre del laberinto)
    (setf result "   ")
    (dotimes (j n result)
      (cond
       ((zerop j)
	(setf u (aref laberinto (- m 1) j))
	(setf cadena (concatenate
		      'string
		      (aref string1 u)
		      (aref string2 u)
		      )
	      )
	)
       (t ;; j > 0
	(setf u (*
		 (aref laberinto (- m 1) j)
		 (aref laberinto (- m 1) (- j 1))
		 )
	      )
	(setf v (aref laberinto (- m 1) j))
	(setf cadena
	      (concatenate 'string
			   (aref string1 u)
			   (aref string2 v)
			   )
	      )
	)
       )
      (setf result (concatenate 'string result cadena)) ;; ir acumulando
      ) ;; fin del dotimes j n ..
    (format t "~a~a~%" result (aref string1 (aref laberinto (- m 1) (- n 1))))
    ) ;; fin del let*
  ) ;; fin de la función


;; no hay control sobre si el problema tiene o no solución
(defun muestra-laberinto (laberinto pos-original)
  (imprime-laberinto-recorrido laberinto (solucion-laberinto laberinto pos-original))
  )

(defun aleat (a b)
  (+ a (random (1+ (- b a))))
  )

;; función para comprobar que el laberinto es correcto y las
;; coordenadas de comienzo son correctas
(defun comprueba-laberinto (laberinto pos-original)
  (let* (
	 (p (array-dimensions laberinto))
	 (m (first p))
	 (n (second p))
	 (cadauxi "")
	 (x 0)
	 (y 0)
	 (u 0)
	 (v 0)
	 (i 0)
	 (j 0)
	 (caso 0)
	 (indc t) ;; indicador de condiciones
	 (ind nil) ;; otro indicador
	 (inde nil) ;; indicador de éxito. Si es t, se cumplen todas las condiciones
	 (lauxi '())
	 )
    (loop while indc do ;; comprobar todas las condiciones
	  (cond
	   ((= caso 0) ;; Comprobar el número de filas y el de columnas
	    (setf caso 1) ;; ir al siguiente a menos que se cumpla lo que sigue
	    (when (or (< m 3) (< n 3)) ;; error, tiene que ser m >= 3 y n >= 3
	      (setf caderro (format nil "Error en las dimensiones del laberinto:~%"))
	      (setf cadauxi (format nil "Tienen que ser No.filas >= 3, No.columnas >= 3~%"))
	      (setf caderro (concatenate 'string caderro cadauxi))
	      (setf cadauxi (format nil "No.filas = ~a, No.columnas = ~a~%" m n))
	      (setf caderro (concatenate 'string caderro cadauxi))
	      (setf indc nil) ;; no seguir comprobando
	      )
	    )
	   ((= caso 1) ;; Comprobar la posición original
	    (setf caso 2)
	    (setf x (primero pos-original))
	    (setf y (segundo pos-original))
	    (when (not (and (>= x 0) (< x m) (>= y 0) (< y n)))
	      (setf caderro (format nil "Error en las coordenadas del punto de comienzo:~%"))
	      (setf cadauxi (format nil "x = ~a, y = ~a~%" x y))
	      (setf caderro (concatenate 'string caderro cadauxi))
	      (setf indc nil) ;; no seguir comprobando
	      )
	    )
	   ((= caso 2) ;; la posición original tiene que ser un blanco
	    (setf caso 3)
	    (setf x (primero pos-original))
	    (setf y (segundo pos-original))
	    (setf u (aref laberinto x y))
	    (when (not (= u 1)) ;; no lo es
	      (setf caderro (format nil "Error en el punto de comienzo. No es blanco.~%"))
	      (setf indc nil) ;; no seguir comprobando
	      )
	    )
	   ((= caso 3) ;; las 4 esquinas tienen que ser muros
	    (setf caso 4)
	    (setf lauxi (list (list 0 0) (list 0 (- n 1)) (list (- m 1) 0) (list (- m 1) (- n 1)))) ;; las cuatro esquinas
	    (setf ind nil) ;; no encontrado ningún error
	    (loop while (not (or (endp lauxi) ind)) do
		  (setf u (primero lauxi))
		  (setf x (primero u))
		  (setf y (segundo u))
		  (setf v (aref laberinto x y))
		  (when (not (zerop v)) ;; no es cero y tendría que serlo
		    (setf caderro (format nil "Error en la esquina (~a,~a)~%" x y))
		    (setf cadauxi (format nil "Debería ser cero y no lo es~%"))
		    (setf caderro (concatenate 'string caderro cadauxi))
		    (setf indc nil) ;; no seguir comprobando
		    (setf ind t) ;; no mirar más esquinas
		    )
		  (setf lauxi (rest lauxi))
		  )
	    )
	   (t ;; el último caso. No puede haber dos blancos seguidos en el perímetro
	    ;; esto para mañana. No olvidar asignar aquí el inde (ndicador de éxito)
	    ;; para saber lo que tenemos que hacer al salir
	    ;; Primero, las filas primera y última
	    (setf indc nil) ;; ya no hay nada más que comprobar. después de esto, salir sin condiciones
	    (setf i 0)
	    (setf ind nil) ;; no encontrado ningún error
	    (loop while (and (< i m) (not ind)) do
		  ;; estoy en la fila i
		  (setf j 0)
		  (loop while (and (< j (- n 1)) (not ind)) do
			(setf u (aref laberinto i j))
			(setf v (aref laberinto i (+ j 1)))
			(when (and (= u 1) (= v 1)) ;; dos blancos seguidos en el borde
			  (setf ind t) ;; salir echando leches. Anotar las posiciones del error
			  (setf x (list i j))
			  (setf y (list i (+ j 1)))
			  (setf caderro (format nil "Dos blancos consecutivos en la fila ~a: ~a, ~a~%" i x y))
			  )
			(setf j (+ j 1))
			)
		  (setf i (+ i (- m 1)))
		  )
	    ;; ahora las columnas primera y última
	    (setf j 0)
	    (loop while (and (< j n) (not ind)) do
		  ;; estoy en la columna j
		  (setf i 0)
		  (loop while (and (< i (- m 1)) (not ind)) do
			(setf u (aref laberinto i j))
			(setf v (aref laberinto (+ i 1) j))
			(when (and (= u 1) (= v 1)) ;; dos blancos seguidos en el borde
			  (setf ind t) ;; salir echando leches. Anotar las posiciones del error
			  (setf x (list i j))
			  (setf y (list (+ i 1) j))
			  (setf caderro (format nil "Dos blancos consecutivos en la columna ~a: ~a, ~a~%" j x y))
			  )
			(setf i (+ i 1))
			)
		  (setf j (+ j (- n 1)))
		  )
	    (when (not ind) ;; todo bien
	      (setf inde t)
	      )
	    )
	   ) ;; fin del cond
	  ) ;; fin del bucle gordo
    inde ;; retornar éste valor
    )
  ) ;; fin de la función

;; esta es la función a la que hay que llamar para resolver el laberinto
(defun solab (laberinto pos-original &optional (recorrido eleccion))
  (let (
	(soluc '())
	(x nil)
	)
    (setf x (comprueba-laberinto laberinto pos-original))
    (cond
     ((not x) ;; error en las condiciones
      (format t caderro) ;; imprimir el error
      )
     (t ;; las condiciones se cumplen
      (setf soluc (solucion-laberinto laberinto pos-original recorrido))
      (if soluc ;; hay solución
	  (imprime-laberinto-recorrido laberinto soluc)
	(format t "Laberinto sin solución.~%")
	)
      )
     )
    )
  )


;; predicado para comprobar si lg = l + algo, es decir:
;; l = (p1 p2 .... pk)
;; lg = (p1 p2 .... pk q1 q2 ... qr)
;; si es así retorna t, y nil en caso contrario
(defun sublizquierdap (l lg)
  (cond
   ((endp l) t)
   ((endp lg) nil)
   ((not (equal (first l) (first lg))) nil)
   (t ;; las dos listas son no vacías y sus primeros elementos son iguales
    (sublizquierdap (rest l) (rest lg))
    )
   )
  )

(defun espartefp (l llistas)
  (if (endp llistas) nil
    (let (
	  (lr (reverse l))
	  (lo llistas)
	  (ind nil)
	  (x '())
	  )
      (loop while (not (or ind (endp lo))) do ;; mientras haya lista o no se encuentra
	    (setf x (reverse (pop lo)))
	    (when (sublizquierdap lr x)
	      (setf ind t) ;; vale
	      )
	    )
      ind ;; retornar éste valor
      )
    )
  ) ;; fin de espartefp


(defun todas-las-soluciones (laberinto pos-original)
  (let (
	(pila '())
	(actual '())
	(lvisit '())
	(camino '())
	(lmovim '())
	(lsoluc '()) ;; lista de soluciones
	(ind-salida nil) ;; cuando sea t, salir sin condiciones
	(inde nil) ;; indicador de éxito o fracaso
	(x '())
	(y '())
	(z '())
	(u '())
	(m 0)
	(n 0)
	)
    (setf u (array-dimensions laberinto))
    (setf m (first u)) ;; número de filas
    (setf n (second u)) ;; número de columnas
    (setf x (list (list pos-original) lvisit camino))
    (push x pila) ;; por aquí hay que comenzar
    (loop while (not (endp pila)) do ;; mientras la pila no sea vacía
	  (setf x (pop pila))
	  (setf lmovim (primero x))
	  (setf lvisit (segundo x))
	  (setf camino (tercero x))
	  (setf actual (pop lmovim))
	  (when (> (length lmovim) 0) ;; aún quedan movimientos
	    (setf y (list lmovim lvisit camino))
	    (push y pila) ;; sigue, que después hay que currar
	    )
	  (setf lvisit (cons actual lvisit))
	  (setf camino (cons actual camino))
	  (setf ind-salida nil)
	  (setf inde nil) ;; de momento, no hay solucuón
	  (loop while (not ind-salida) do ;; mientras haya algo que hacer
		(cond
		 ((esta-resuelto-labp actual pos-original m n)
		  (setf ind-salida t)
		  (setf inde t)
		  )
		 (t ;; no resuelto, hay que trabajar.
		  ;; Busca los adyacentes a actual no visitados
		  (setf x (lista-de-adyacentes-no-visitados actual lvisit laberinto))
		  (cond
		   ((endp x) ;; no hay ninguno, hay que retroceder
		    (pop camino) ;; quitar el último de camino
		    (cond
		     ((endp camino) ;; no se puede retroceder, problema sin solución
		      (setf ind-salida t)
		      (setf inde nil)
		      )
		     (t ;; retroceso propiamente dicho
		      (setf actual (first camino)) ;; retroceder
		      )
		     )
		    )
		   (t ;; lista de adyacentes no vacía
		    (setf u (pop x)) ;; el primer movimiento
		    (when (> (length x) 0) ;; aparte de u hay otros
		      (setf y (list x lvisit camino))
		      (push y pila) ;; sigue, que después hay que currar
		      )
		    (setf lvisit (cons u lvisit))
		    (setf camino (cons u camino))
		    (setf actual u)
		    )
		   )
		  )
		 )
		) ;; fin de loop
	  (when inde ;; ha habido suerte, se ha encontrado una solución
	    (when (not (member camino lsoluc :test #'equal))
	      (push camino lsoluc)
	      )
	    )
	  ) ;; fin del loop de la pila
    lsoluc ;; retornar esto
    ) ;; fin del let
  ) ;; fin de la función

(defun imprime-lista (lista)
  (dolist (x lista)
    (format t "~a~%" x)
    )
  )

;; esta es la función a la que hay que llamar para mostrar todas
;; las soluciones gráficas del laberinto
(defun tsolab (laberinto pos-original)
  (let (
	(lsoluc '())
	(x nil)
	)
    (setf x (comprueba-laberinto laberinto pos-original))
    (cond
     ((not x) ;; error en las condiciones
      (format t caderro) ;; imprimir el error
      )
     (t ;; las condiciones se cumplen
      (setf lsoluc (todas-las-soluciones laberinto pos-original))
      (if lsoluc ;; hay solución
	  (mapcar #'(lambda(x) (imprime-laberinto-recorrido laberinto x)) lsoluc)
	(format t "Laberinto sin solución.~%")
	)
      )
     )
    )
  )

;; esta es la función a la que hay que llamar para mostrar todas
;; las soluciones en forma de camino del laberinto
(defun tsolabl (laberinto pos-original)
  (let (
	(lsoluc '())
	(vret nil)
	(x nil)
	)
    (setf x (comprueba-laberinto laberinto pos-original))
    (when x
      (setf lsoluc (todas-las-soluciones laberinto pos-original))
       (when (not (endp lsoluc)) ;; hay solución
	 (setf vret (mapcar #'(lambda(x) (reverse x)) lsoluc))
	 )
       )
    vret
    )
  )

;; De todas las posibles soluciones, esta función retorna la(s) mínimas
(defun solucion-minima (laberinto pos-original)
  (let (
	(lsoluc '())
	(vret nil)
	(x nil)
	(lmin 0)
	)
    (setf x (comprueba-laberinto laberinto pos-original))
    (when x
      (setf lsoluc (todas-las-soluciones laberinto pos-original))
       (when (not (endp lsoluc)) ;; hay solución
	 (setf lmin (apply #'min (mapcar #'length lsoluc)))
	 (setf vret (remove-if #'(lambda(x) (not (= (length x) lmin))) lsoluc))
	 )
       )
    vret
    )
  )

;; Ésta función muestra los caminos mínimos del laberinto
(defun muestra-minimos (laberinto pos-original)
  (let (
	(lsoluc '())
	(vret nil)
	(x nil)
	(lmin 0)
	)
    (setf x (comprueba-laberinto laberinto pos-original))
    (when x
      (setf lsoluc (solucion-minima laberinto pos-original))
       (when (not (endp lsoluc)) ;; hay soluciones mínimas
	 (mapcar #'(lambda(x) (imprime-laberinto-recorrido laberinto x)) lsoluc)
	 )
       )
    )
  )
